@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: doc2html.c,v 1.10 2017/05/14 04:08:12 markisch Exp $"); }
-#endif
-
 /* GNUPLOT - doc2html.c */
 
 /*[
@@ -70,8 +66,8 @@ static TBOOLEAN debug = FALSE;
 static char path[PATH_MAX];
 static const char name[] = "wgnuplot";
 
-void convert __PROTO((FILE *, FILE *, FILE *, FILE *));
-void process_line __PROTO((char *, FILE *, FILE *, FILE *));
+void convert(FILE *, FILE *, FILE *, FILE *);
+void process_line(char *, FILE *, FILE *, FILE *);
 
 int
 main (int argc, char **argv)
@@ -352,7 +348,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 /* NB: don't set inkey here */
 #endif
                 if (debug)
-                    fprintf(stderr,"keyword defintion: \"%s\" on line %d.\n", line2 + 1, line_count);
+                    fprintf(stderr,"keyword definition: \"%s\" on line %d.\n", line2 + 1, line_count);
             }
 	    break;
     case '@':{			/* start/end table */
@@ -492,7 +488,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                     strcpy(location, name);
 
 #ifndef SPLIT_FILES
-                /* let hhc auomatically split contents */
+                /* let hhc automatically split contents */
                 if (!startpage) {
                     fprintf(b, "<OBJECT type=\"application/x-oleobject\" classid=\"clsid:1e2a7bd0-dab9-11d0-b93a-00c04fc99f9e\">\n");
 	            fprintf(b, "<param name=\"New HTML file\" value=\"%s.html\">\n", location);
@@ -503,15 +499,16 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
 #else
 		/* split contents manually */
                 if (!startpage) {
-		    char newfile[PATH_MAX];
+		    char newfile[PATH_MAX] = "";
 
                     /* close current file */
 		    footer(b);
 		    fclose(b);
 
                     /* open new file */
-                    sprintf(newfile, "%s%s.html", path, location);
-                    /* fprintf(stderr, "%s\n", newfile); */
+		    strcat(newfile,path);
+		    strncat(newfile,location,PATH_MAX-strlen(newfile)-6);
+		    strcat(newfile,".html");
                     if (!(b = fopen(newfile, "w"))) {
                         fprintf(stderr, "%s: Can't open %s for writing\n",
                             "doc2html", newfile);
