@@ -1,7 +1,3 @@
-/*
- * $Id: eval.h,v 1.51 2016/09/19 04:40:30 sfeam Exp $
- */
-
 /* GNUPLOT - eval.h */
 
 /*[
@@ -67,11 +63,16 @@ enum operators {
 #endif
 
     /* functions specific to using spec */
-    COLUMN, STRINGCOLUMN
+    COLUMN, STRINGCOLUMN, STRCOL, COLUMNHEAD
 };
 #define is_jump(operator) \
     ((operator) >=(int)JUMP && (operator) <(int)SF_START)
 
+enum int64_overflow {
+    INT64_OVERFLOW_IGNORE=0, INT64_OVERFLOW_TO_FLOAT, \
+    INT64_OVERFLOW_UNDEFINED, INT64_OVERFLOW_NAN
+};
+
 /* user-defined function table entry */
 typedef struct udft_entry {
     struct udft_entry *next_udf; /* pointer to next udf in linked list */
@@ -103,7 +104,7 @@ typedef union argument {
 
 
 /* This type definition has to come after union argument has been declared. */
-typedef void (*FUNC_PTR) __PROTO((union argument *arg));
+typedef void (*FUNC_PTR)(union argument *arg);
 
 /* standard/internal function table entry */
 typedef struct ft_entry {
@@ -127,7 +128,7 @@ struct at_type {
 
 /* Variables of eval.c needed by other modules: */
 
-extern const struct ft_entry GPFAR ft[]; /* The table of builtin functions */
+extern const struct ft_entry ft[]; /* The table of builtin functions */
 extern struct udft_entry *first_udf; /* user-def'd functions */
 extern struct udvt_entry *first_udv; /* user-def'd variables */
 extern struct udvt_entry udv_pi; /* 'pi' variable */
@@ -135,55 +136,57 @@ extern struct udvt_entry *udv_NaN; /* 'NaN' variable */
 extern struct udvt_entry **udv_user_head; /* first udv that can be deleted */
 extern TBOOLEAN undefined;
 
+extern enum int64_overflow overflow_handling;
+
 /* Prototypes of functions exported by eval.c */
 
-double gp_exp __PROTO((double x));
+double gp_exp(double x);
 
 /* HBB 20010726: Moved these here, from util.h. */
-double real __PROTO((struct value *));
-int real_int __PROTO((struct value *val));
-double imag __PROTO((struct value *));
-double magnitude __PROTO((struct value *));
-double angle __PROTO((struct value *));
-struct value * Gcomplex __PROTO((struct value *, double, double));
-struct value * Ginteger __PROTO((struct value *, int));
-struct value * Gstring __PROTO((struct value *, char *));
-struct value * pop_or_convert_from_string __PROTO((struct value *));
-struct value * gpfree_string __PROTO((struct value *a));
-void gpfree_array __PROTO((struct value *a));
+double real(struct value *);
+double imag(struct value *);
+double magnitude(struct value *);
+double angle(struct value *);
+struct value * Gcomplex(struct value *, double, double);
+struct value * Ginteger(struct value *, intgr_t);
+struct value * Gstring(struct value *, char *);
+struct value * pop_or_convert_from_string(struct value *);
+void free_value(struct value *a);
+void gpfree_string(struct value *a);
+void gpfree_array(struct value *a);
 
-void reset_stack __PROTO((void));
-void check_stack __PROTO((void));
-TBOOLEAN more_on_stack __PROTO((void));
-struct value *pop __PROTO((struct value *x));
-void push __PROTO((struct value *x));
-void int_check __PROTO((struct value * v));
+void reset_stack(void);
+void check_stack(void);
+TBOOLEAN more_on_stack(void);
+struct value *pop(struct value *x);
+void push(struct value *x);
+void int_check(struct value * v);
 
-void f_bool __PROTO((union argument *x));
-void f_jump __PROTO((union argument *x));
-void f_jumpz __PROTO((union argument *x));
-void f_jumpnz __PROTO((union argument *x));
-void f_jtern __PROTO((union argument *x));
+void f_bool(union argument *x);
+void f_jump(union argument *x);
+void f_jumpz(union argument *x);
+void f_jumpnz(union argument *x);
+void f_jtern(union argument *x);
 
-void execute_at __PROTO((struct at_type *at_ptr));
-void evaluate_at __PROTO((struct at_type *at_ptr, struct value *val_ptr));
-void real_free_at __PROTO((struct at_type *at_ptr));
-struct udvt_entry * add_udv_by_name __PROTO((char *key));
-struct udvt_entry * get_udv_by_name __PROTO((char *key));
-void del_udv_by_name __PROTO(( char *key, TBOOLEAN isWildcard ));
-void clear_udf_list __PROTO((void));
+void execute_at(struct at_type *at_ptr);
+void evaluate_at(struct at_type *at_ptr, struct value *val_ptr);
+void real_free_at(struct at_type *at_ptr);
+struct udvt_entry * add_udv_by_name(char *key);
+struct udvt_entry * get_udv_by_name(char *key);
+void del_udv_by_name( char *key, TBOOLEAN isWildcard );
+void clear_udf_list(void);
 
 /* update GPVAL_ variables available to user */
-void update_gpval_variables __PROTO((int from_plot_command));
+void update_gpval_variables(int from_plot_command);
 /* note: the routines below work for any variable name, not just those beginning GPVAL_ */
-void fill_gpval_string __PROTO((char *var, const char *value));
-void fill_gpval_integer __PROTO((char *var, int value));
-void fill_gpval_float __PROTO((char *var, double value));
-void fill_gpval_complex __PROTO((char *var, double areal, double aimag));
+void fill_gpval_string(char *var, const char *value);
+void fill_gpval_integer(char *var, intgr_t value);
+void fill_gpval_float(char *var, double value);
+void fill_gpval_complex(char *var, double areal, double aimag);
 
 /* C-callable versions of internal gnuplot functions word() and words() */
-char * gp_word __PROTO((char *string, int i));
-int gp_words __PROTO((char *string));
+char * gp_word(char *string, int i);
+int gp_words(char *string);
 
 /* Wrap real_free_at in a macro */
 #define free_at(at_ptr) \
