@@ -62,8 +62,8 @@ extern "C" {
 	#include "alloc.h"     // for gp_alloc
 	#include "parse.h"     // for real_expression
 	#include "axis.h"
-#ifdef WIN32
-	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh, stdin_pipe_reader
+#ifdef _WIN32
+	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh
 	#include "win/wgnuplib.h" // for TextStartEditing, TextStopEditing
 	#include "win/wtext.h"    // for kbhit, getchar
 	#include <io.h>           // for isatty
@@ -170,6 +170,8 @@ static int  qt_optionFontSize = 9;
 static double qt_optionDashLength = 1.0;
 static double qt_optionLineWidth = 1.0;
 
+static int qt_optionctrlq = -1;	// tristate -1 = not set 0 = false 1 = true 
+
 /* Encapsulates all Qt options that have a constructor and destructor. */
 struct QtOption {
     QtOption()
@@ -179,14 +181,14 @@ struct QtOption {
     QString FontName;
     QString Title;
     QString Widget;
-	QPoint  position;
+    QPoint  position;
 };
 static QtOption* qt_option = NULL;
 
 static void ensureOptionsCreated()
 {
 	if (!qt_option)
-	    qt_option = new QtOption();
+		qt_option = new QtOption();
 }
 
 static bool qt_setPosition = false;
@@ -228,7 +230,7 @@ QPoint qt_gnuplotCoord(int x, int y)
 }
 
 #ifndef GNUPLOT_QT
-# ifdef WIN32
+# ifdef _WIN32
 #  define GNUPLOT_QT "gnuplot_qt.exe"
 # else
 #  define GNUPLOT_QT "gnuplot_qt"
@@ -243,7 +245,7 @@ void execGnuplotQt()
 	if (path)
 		filename = QString(path);
 	if (filename.isEmpty()) {
-#ifdef WIN32
+#ifdef _WIN32
 		filename = QCoreApplication::applicationDirPath();
 #else
 		filename = QT_DRIVER_DIR;
@@ -263,6 +265,7 @@ void execGnuplotQt()
 #endif
 	} else {
 		fprintf(stderr, "Could not start gnuplot_qt with path %s\n", filename.toUtf8().data());
+		fprintf(stderr, "Did you set environmental variable GNUPLOT_DRIVER_DIR?\n");
 	}
 }
 
@@ -426,7 +429,7 @@ bool qt_processTermEvent(gp_event_t* event)
 		paused_for_mouse = 0;
 		return true;
 	}
-	if ((event->type == GE_reset))
+	if (event->type == GE_reset)
 	{
 		paused_for_mouse = 0;
 		return true;
@@ -455,7 +458,7 @@ void qt_init()
 	// The creation of a QApplication mangled our locale settings
 #ifdef HAVE_LOCALE_H
 	setlocale(LC_NUMERIC, "C");
-	setlocale(LC_TIME, current_locale);
+	setlocale(LC_TIME, time_locale);
 #endif
 
 	qt->out.setVersion(QDataStream::Qt_4_4);
@@ -561,7 +564,14 @@ void qt_graphics()
 #endif
 	qt->out << GEActivate;
 	qt->out << GETitle << qt_option->Title;
-	qt->out << GESetCtrl << qt_optionCtrl;
+
+	// We used to always send true or false; now it's a tristate
+	// only sent if there was an explicit set/unset in the "set term" command
+	if (qt_optionctrlq >= 0) {
+		qt->out << GESetCtrl << qt_optionCtrl;
+		qt_optionctrlq = -1;
+	}
+
 	qt->out << GESetWidgetSize << QSize(term->xmax, term->ymax)/qt_oversampling;
 	// Initialize the scene
 	qt->out << GESetSceneSize << QSize(term->xmax, term->ymax)/qt_oversampling;
@@ -629,7 +639,6 @@ void qt_text_wrapper()
 
 void qt_reset()
 {
-	/// @todo
 }
 
 void qt_move(unsigned int x, unsigned int y)
@@ -647,7 +656,7 @@ void qt_enhanced_flush()
 	qt->out << GEEnhancedFlush << qt->enhancedFontName << qt->enhancedFontSize
 		<< (int)qt->enhancedFontStyle << (int)qt->enhancedFontWeight
 		<< qt->enhancedBase << qt->enhancedWidthFlag << qt->enhancedShowFlag
-		<< qt->enhancedOverprint 
+		<< qt->enhancedOverprint
 		<< qt->codec->toUnicode(qt->enhancedText);
 	qt->enhancedText.clear();
 }
@@ -670,9 +679,9 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 
 	// Baseline correction.  Surely Qt itself provides this somehow?
 	if (qt_max_pos_base < base)
-	    qt_max_pos_base = base;
+		qt_max_pos_base = base;
 	if (qt_max_neg_base > base)
-	    qt_max_neg_base = base;
+		qt_max_neg_base = base;
 
 	// strip Bold or Italic property out of font name
 	QString tempname = fontname;
@@ -687,7 +696,7 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 	int sep = tempname.indexOf(":");
 	if (sep >= 0)
 		tempname.truncate(sep);
-	
+
 	// Blank font name means keep using the previous font
 	if (!tempname.isEmpty())
 		qt->enhancedFontName = tempname;
@@ -754,12 +763,13 @@ void qt_linetype(int lt)
 		qt->out << GEPenStyle << Qt::DotLine;
 	else if (lt == LT_NODRAW)
 		qt->out << GEPenStyle << Qt::NoPen;
-	else 
+	else
 		qt->out << GEPenStyle << Qt::SolidLine;
 
-	if (lt == LT_BACKGROUND)
+	if (lt == LT_BACKGROUND) {
+		/* FIXME: Add parameter to this API to set the background color from the gnuplot end */
 		qt->out << GEBackgroundColor;
-	else if (lt > LT_NODRAW)
+	} else if (lt > LT_NODRAW)
 		qt->out << GEPenColor << qt_colorList[lt % 9 + 3];
 }
 
@@ -829,7 +839,7 @@ int qt_set_font(const char* font)
 
 	/* Optimize by leaving early if there is no change */
 	if (qt->currentFontSize == qt_previousFontSize
-	&&  qt->currentFontName == qt->currentFontName) {
+	&&  qt->currentFontName == qt_previousFontName) {
 		return 1;
 	}
 
@@ -874,8 +884,14 @@ int qt_text_angle(int angle)
 
 void qt_fillbox(int style, unsigned int x, unsigned int y, unsigned int width, unsigned int height)
 {
+	// If y >= ymax then no box is drawn. So we clip.
+	unsigned int ytop = y + height;
+	if (ytop >= term->ymax) {
+	    ytop = term->ymax-1;
+	    height = ytop - y;
+	}
 	qt->out << GEBrushStyle << style;
-	qt->out << GEFillBox << QRect(qt_termCoord(x, y + height), QSize(width, height)/qt_oversampling);
+	qt->out << GEFillBox << QRect(qt_termCoord(x, ytop), QSize(width, height)/qt_oversampling);
 }
 
 int qt_make_palette(t_sm_palette* palette)
@@ -935,8 +951,8 @@ void qt_image(unsigned int M, unsigned int N, coordval* image, gpiPoint* corner,
 // box, with \r separating text above and below the point.
 void qt_put_tmptext(int n, const char str[])
 {
-    if (!qt)
-        return;
+	if (!qt)
+		return;
 
 	if (n == 0)
 		qt->out << GEStatusText << QString(str);
@@ -950,8 +966,8 @@ void qt_put_tmptext(int n, const char str[])
 
 void qt_set_cursor(int c, int x, int y)
 {
-    if (!qt)
-        return;
+	if (!qt)
+		return;
 
 	// Cancel zoombox when Echap is pressed
 	if (c == 0)
@@ -998,7 +1014,7 @@ void qt_set_clipboard(const char s[])
 int qt_waitforinput(int options)
 {
 #ifdef USE_MOUSE
-#ifndef WIN32
+#ifndef _WIN32
 	fd_set read_fds;
 	struct timeval one_msec;
 	int stdin_fd  = fileno(stdin);
@@ -1100,7 +1116,7 @@ int qt_waitforinput(int options)
 
 	return getchar();
 
-#else // Windows console and wgnuplot
+#else // _WIN32, Windows console and wgnuplot
 #ifdef WGP_CONSOLE
 	int fd = fileno(stdin);
 #endif
@@ -1121,7 +1137,7 @@ int qt_waitforinput(int options)
 	if (options != TERM_ONLY_CHECK_MOUSING) { // NOTE: change this if used also for the caca terminal
 #ifdef WGP_CONSOLE
 		if (!isatty(fd))
-			h[0] = CreateThread(NULL, 0, stdin_pipe_reader, NULL, 0, NULL);
+			h[0] = (HANDLE)_get_osfhandle(fd);
 		else
 #endif
 			h[0] = GetStdHandle(STD_INPUT_HANDLE);
@@ -1147,7 +1163,7 @@ int qt_waitforinput(int options)
 
 	do {
 		int waitResult = -1;
-		
+
 #ifndef WGP_CONSOLE
 		// Process pending key events of the text console
 		if (kbhit() && (options != TERM_ONLY_CHECK_MOUSING))
@@ -1158,21 +1174,21 @@ int qt_waitforinput(int options)
 		if ((idx_socket != -1) && // (qt != NULL)) &&
 			(qt->socket.waitForReadyRead(0)) && (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)))
 			waitResult = idx_socket; // data already available
-		
-		// Wait for a new event 
+
+		// Wait for a new event
 		if ((waitResult == -1) && (options != TERM_ONLY_CHECK_MOUSING))
 			waitResult = MsgWaitForMultipleObjects(idx, h, FALSE, INFINITE, QS_ALLINPUT);  // wait for new data
-		
+
 		if ((waitResult == idx_stdin) && (idx_stdin != -1)) { // console windows or caca terminal (TBD)
 #ifdef WGP_CONSOLE
 			if (!isatty(fd)) {
-				DWORD dw;
-
-				GetExitCodeThread(h[0], &dw);
-				CloseHandle(h[0]);
-				c = dw;
+				unsigned char ch;
+				if (fread(&ch, 1, 1, stdin) == 1)
+					c = ch;
+				else
+					c = EOF;
 				quitLoop = true;
-			} else 
+			} else
 #endif
 			{
 				c = ConsoleReadCh();
@@ -1206,7 +1222,7 @@ int qt_waitforinput(int options)
 					}
 				}
 			}
-			// If the native pipe handle signalled new data, but the QtLocalSocket 
+			// If the native pipe handle signalled new data, but the QtLocalSocket
 			// object has no data available, release the CPU for a little while.
 			if (size == 0)
 				Sleep(100);
@@ -1240,17 +1256,18 @@ int qt_waitforinput(int options)
 		}
 	} while (!quitLoop);
 
-
 #ifndef WGP_CONSOLE
 	if (options != TERM_ONLY_CHECK_MOUSING)
 		TextStopEditing(&textwin);
-	
+
 	// This happens if neither the qt queue is alive, nor there is a console window.
 	if ((options != TERM_ONLY_CHECK_MOUSING) && !waitOK)
 		return getchar();
+#else
+	(void) waitOK;
 #endif
 	return c;
-#endif // WIN32
+#endif // _WIN32
 #else
 	return getchar();
 #endif // USE_MOUSE
@@ -1273,9 +1290,9 @@ void qt_atexit()
 	else
 		qt->out << GEExit;
 	qt_flushOutBuffer();
-        
-        delete qt;
-        qt = NULL;
+
+	delete qt;
+	qt = NULL;
 
 	delete qt_option;
 	qt_option = NULL;
@@ -1353,7 +1370,7 @@ void qt_options()
 	bool set_linewidth = false;
 	int previous_WindowId = qt_optionWindowId;
 
-#ifndef WIN32
+#ifndef _WIN32
 	if (term_interlock != NULL && term_interlock != (void *)qt_init) {
 		term = NULL;
 		int_error(NO_CARET, "The qt terminal cannot be used in a wxt session");
@@ -1520,8 +1537,9 @@ void qt_options()
 		qt_setPosition = true;
 	}
 
+	termOptions += " font \"" + fontSettings + '"';
+
 	if (set_enhanced) termOptions += qt_optionEnhanced ? " enhanced" : " noenhanced";
-	                  termOptions += " font \"" + fontSettings + '"';
 	if (set_linewidth) termOptions += " linewidth " + QString::number(qt_optionLineWidth);
 	if (set_dashlength) termOptions += " dashlength " + QString::number(qt_optionDashLength);
 	if (set_widget)   termOptions += " widget \"" + qt_option->Widget + '"';
@@ -1529,18 +1547,24 @@ void qt_options()
 	if (set_raise)    termOptions += qt_optionRaise ? " raise" : " noraise";
 	if (set_ctrl)     termOptions += qt_optionCtrl ? " ctrl" : " noctrl";
 
+	/// Only send the ctrlQ option if it is explicitly given in "set term qt {no}ctrlq"
+	if (set_ctrl)
+	    qt_optionctrlq = qt_optionCtrl;
+	else
+	    qt_optionctrlq = -1;
+
 	/// @bug change Utf8 to local encoding
 	strncpy(term_options, termOptions.toUtf8().data(), MAX_LINE_LEN);
 }
 
 void qt_layer( t_termlayer syncpoint )
 {
-    static int current_plotno = 0;
+	static int current_plotno = 0;
 	if (!qt)
 		return;
 
-    /* We must ignore all syncpoints that we don't recognize */
-    switch (syncpoint) {
+	/* We must ignore all syncpoints that we don't recognize */
+	switch (syncpoint) {
 	case TERM_LAYER_BEFORE_PLOT:
 		current_plotno++;
 		qt->out << GEPlotNumber << current_plotno; break;
@@ -1564,9 +1588,9 @@ void qt_layer( t_termlayer syncpoint )
 		qt->out << GELayer << QTLAYER_BEFORE_ZOOM; break;
 	case TERM_LAYER_3DPLOT:
 		qt_is_3Dplot = true; break;
-    	default:
+	default:
 		break;
-    }
+	}
 }
 
 void qt_hypertext( int type, const char *text )
@@ -1575,15 +1599,16 @@ void qt_hypertext( int type, const char *text )
 		qt->out << GEHypertext << qt->codec->toUnicode(text);
 }
 
-#ifdef EAM_BOXED_TEXT
 void qt_boxed_text(unsigned int x, unsigned int y, int option)
 {
-	if (option == TEXTBOX_MARGINS)
-	    qt->out << GETextBox << QPointF((double)x/(100*qt_oversamplingF), (double)y/(100*qt_oversamplingF)) << option;
-	else
+	if (option == TEXTBOX_MARGINS) {
+	    // slightly largin y margin to allow for descenders
+	    double xmargin = (double)x/(100*qt_oversamplingF);
+	    double ymargin = (double)y/(100*qt_oversamplingF) + 0.75/qt_oversamplingF;
+	    qt->out << GETextBox << QPointF(xmargin, ymargin) << option;
+	} else
 	    qt->out << GETextBox << qt_termCoordF(x, y) << option;
 }
-#endif
 
 void qt_modify_plots(unsigned int ops, int plotno)
 {
