@@ -40,7 +40,7 @@
  *   Msdos port and some enhancements:
  *     Gershon Elber and many others.
  *
- *   Adapted to work with UTF-8 enconding.
+ *   Adapted to work with UTF-8 encoding.
  *     Ethan A Merritt  April 2011
  */
 
@@ -53,6 +53,7 @@
 #include "gp_hist.h"
 #include "plot.h"
 #include "util.h"
+#include "encoding.h"
 #include "term_api.h"
 #ifdef HAVE_WCHAR_H
 #include <wchar.h>
@@ -74,7 +75,14 @@ getc_wrapper(FILE* fp)
     while (1) {
 	errno = 0;
 #ifdef USE_MOUSE
-	if (term && term->waitforinput && interactive) {
+	/* EAM June 2020:
+	 * For 20 years this was conditional on interactive.
+	 *    if (term && term->waitforinput && interactive)
+	 * Now I am suspicious that this was the cause of dropped
+	 * characters when mixing piped input with mousing,
+	 * e.g. Bugs 2134, 2279
+	 */
+	if (term && term->waitforinput) {
 	    c = term->waitforinput(0);
 	}
 	else
@@ -98,7 +106,7 @@ getc_wrapper(FILE* fp)
 /*
  * The signal handler of libreadline sets a flag when SIGTSTP is received
  * but does not suspend until this flag is checked by other library
- * routines.  Since gnuplot's term->waitforinput() + getc_wrapper() 
+ * routines.  Since gnuplot's term->waitforinput() + getc_wrapper()
  * replace these other routines, we must do the test and suspend ourselves.
  */
 void
@@ -148,7 +156,7 @@ wrap_readline_signal_handler()
  * ^U kills the entire line
  * ^W deletes previous full or partial word
  * ^V disables interpretation of the following key
- * LF and CR return the entire line regardless of the cursor postition
+ * LF and CR return the entire line regardless of the cursor position
  * DEL deletes previous or current character (configuration dependent)
  * TAB will perform filename completion
  * ^R start a backward-search of the history
@@ -162,7 +170,7 @@ wrap_readline_signal_handler()
 # include <sys/ioctl.h>
 #endif
 
-/* replaces the previous klugde in configure */
+/* replaces the previous kludge in configure */
 #if defined(HAVE_TERMIOS_H) && defined(HAVE_TCGETATTR)
 # define TERMIOS
 #else /* not HAVE_TERMIOS_H && HAVE_TCGETATTR */
@@ -256,7 +264,7 @@ static char term_chars[NCCS];
 static int term_set = 0;	/* =1 if rl_termio set */
 
 #define special_getc() ansi_getc()
-static int ansi_getc __PROTO((void));
+static int ansi_getc(void);
 #define DEL_ERASES_CURRENT_CHAR
 
 #else /* MSDOS or _WIN32 */
@@ -272,7 +280,7 @@ static int ansi_getc __PROTO((void));
 static int win_getch(void);
 #  else
     /* The wgnuplot text window will suppress intermediate
-       screen updates in 'suspend' mode and only redraw the 
+       screen updates in 'suspend' mode and only redraw the
        input line after 'resume'. */
 #   define SUSPENDOUTPUT TextSuspend(&textwin)
 #   define RESUMEOUTPUT TextResume(&textwin)
@@ -332,23 +340,23 @@ static const char search_prompt2[] = "': ";
 static struct hist * search_result = NULL;
 static int search_result_width = 0;	/* on-screen width of the search result */
 
-static void fix_line __PROTO((void));
-static void redraw_line __PROTO((const char *prompt));
-static void clear_line __PROTO((const char *prompt));
-static void clear_eoline __PROTO((const char *prompt));
-static void delete_previous_word __PROTO((void));
-static void copy_line __PROTO((char *line));
-static void set_termio __PROTO((void));
-static void reset_termio __PROTO((void));
-static int user_putc __PROTO((int ch));
-static int user_puts __PROTO((char *str));
-static int backspace __PROTO((void));
-static void extend_cur_line __PROTO((void));
-static void step_forward __PROTO((void));
-static void delete_forward __PROTO((void));
-static void delete_backward __PROTO((void));
-static int char_seqlen __PROTO((void));
-#if defined(HAVE_DIRENT_H) || defined(_WIN32)
+static void fix_line(void);
+static void redraw_line(const char *prompt);
+static void clear_line(const char *prompt);
+static void clear_eoline(const char *prompt);
+static void delete_previous_word(void);
+static void copy_line(char *line);
+static void set_termio(void);
+static void reset_termio(void);
+static int user_putc(int ch);
+static int user_puts(char *str);
+static int backspace(void);
+static void extend_cur_line(void);
+static void step_forward(void);
+static void delete_forward(void);
+static void delete_backward(void);
+static int char_seqlen(void);
+#if defined(HAVE_DIRENT)
 static char *fn_completion(size_t anchor_pos, int direction);
 static void tab_completion(TBOOLEAN forward);
 #endif
@@ -621,7 +629,7 @@ extend_cur_line()
 }
 
 
-#if defined(HAVE_DIRENT_H) || defined(_WIN32)
+#if defined(HAVE_DIRENT)
 static char *
 fn_completion(size_t anchor_pos, int direction)
 {
@@ -678,7 +686,7 @@ fn_completion(size_t anchor_pos, int direction)
 	    path = gp_strdup("");
 	}
 
-	/* seperate directory and (partial) file directory name */
+	/* separate directory and (partial) file directory name */
 	t = strrchr(path, DIRSEP1);
 #if DIRSEP2 != NUL
 	if (t == NULL) t = strrchr(path, DIRSEP2);
@@ -809,7 +817,7 @@ tab_completion(TBOOLEAN forward)
     last_completion_len = completion_len;
 }
 
-#endif /* HAVE_DIRENT_H || _WIN32 */
+#endif /* HAVE_DIRENT */
 
 
 char *
@@ -1009,7 +1017,7 @@ readline(const char *prompt)
 		    step_forward();
 		}
 		break;
-#if defined(HAVE_DIRENT_H) || defined(_WIN32)
+#if defined(HAVE_DIRENT)
 	    case 011:		/* ^I / TAB */
 		tab_completion(TRUE); /* next tab completion */
 		break;
@@ -1057,7 +1065,7 @@ readline(const char *prompt)
 		break;
 	    case 004:		/* ^D */
 		/* Also catch asynchronous termination signal on Windows */
-		if (max_pos == 0 && terminate_flag) {
+		if (max_pos == 0 || terminate_flag) {
 		    reset_termio();
 		    return NULL;
 		}
@@ -1105,14 +1113,14 @@ readline(const char *prompt)
 	    default:
 		break;
 	    }
-	} 
+	}
 
 	} else {  /* search-mode */
 #ifdef VERASE
 	    if (cur_char == term_chars[VERASE]) {	/* ^H */
 		delete_backward();
 		do_search(-1);
-	    } else 
+	    } else
 #endif /* VERASE */
 	    {
 	    switch (cur_char) {
@@ -1224,7 +1232,7 @@ switch_prompt(const char * old_prompt, const char * new_prompt)
     putc('\r', stderr);
     fputs(new_prompt, stderr);
     cur_pos = 0;
-    
+
     /* erase remainder of previous prompt */
     len = GPMAX((int)strlen(old_prompt) - (int)strlen(new_prompt), 0);
     for (i = 0; i < len; i++)
@@ -1398,7 +1406,10 @@ ansi_getc()
     int c;
 
 #ifdef USE_MOUSE
-    if (term && term->waitforinput && interactive)
+    /* EAM June 2020 why only interactive?
+     * if (term && term->waitforinput && interactive)
+     */
+    if (term && term->waitforinput)
 	c = term->waitforinput(0);
     else
 #endif
@@ -1460,6 +1471,7 @@ msdos_getch()
     int c;
 
 #ifdef DJGPP
+    /* no need to handle mouse input here: it's done in term->text() */
     int ch = getkey();
     c = (ch & 0xff00) ? 0 : ch & 0xff;
 #elif defined (OS2)
@@ -1511,6 +1523,9 @@ msdos_getch()
 	case 83:		/* Delete */
 	    c = 0177;
 	    break;
+	case 15:		/* BackTab / Shift-Tab */
+	    c = 034;	/* FS: remap to non-standard code for tab-completion */
+	    break;
 	default:
 	    c = 0;
 	    break;
