@@ -90,6 +90,7 @@
 #include "misc.h"
 #include "multiplot.h"
 #include "readline.h"
+#include "encoding.h"
 
 #ifdef USE_MOUSE
 #include "mouse.h"
@@ -98,20 +99,15 @@
 /* even if the interactive gnuplot session itself cannot.      */
 long mouse_mode = 0;
 char* mouse_alt_string = NULL;
+#define MOUSE_COORDINATES_FUNCTION 8	/* Normally an enum in mouse.h */
 #endif
 
 #ifdef _WIN32
 # include "win/winmain.h"
 # include "win/wcommon.h"
-# ifdef __MSC__
-#  include <malloc.h>
-#  include <io.h>
-# else
-#  include <alloc.h>
-# endif                         /* MSC */
-#endif /* _WIN32 */
+#endif
 
-static int termcomp __PROTO((const generic * a, const generic * b));
+static int termcomp(const generic * a, const generic * b);
 
 /* Externally visible variables */
 /* the central instance: the current terminal's interface structure */
@@ -129,6 +125,7 @@ FILE *gpoutfile;
 */
 FILE *gppsfile = 0;
 char *PS_psdir = NULL;
+char *PS_fontpath = NULL;
 
 /* true if terminal has been initialized */
 TBOOLEAN term_initialised;
@@ -142,6 +139,7 @@ TBOOLEAN monochrome = FALSE;
 
 /* true if in multiplot mode */
 TBOOLEAN multiplot = FALSE;
+int multiplot_count = 0;
 
 /* text output encoding, for terminals that support it */
 enum set_encoding_id encoding;
@@ -177,10 +175,6 @@ const char *arrow_head_names[4] =
     {"nohead", "head", "backhead", "heads"};
 
 enum { IPC_BACK_UNUSABLE = -2, IPC_BACK_CLOSED = -1 };
-#ifdef PIPE_IPC
-/* HBB 20020225: currently not used anywhere outside term.c */
-static SELECT_TYPE_ARG1 ipc_back_fd = IPC_BACK_CLOSED;
-#endif
 
 /* resolution in dpi for converting pixels to size units */
 int gp_resolution = 72;
@@ -191,6 +185,7 @@ char *enhanced_cur_text = NULL;
 double enhanced_fontscale = 1.0;
 char enhanced_escape_format[16] = "";
 double enhanced_max_height = 0.0, enhanced_min_height = 0.0;
+#define ENHANCED_TEXT_MAX (&enhanced_text[MAX_LINE_LEN])
 /* flag variable to disable enhanced output of filenames, mainly. */
 TBOOLEAN ignore_enhanced_text = FALSE;
 
@@ -218,32 +213,32 @@ static double term_pointsize=1;
 
 /* Internal prototypes: */
 
-static void term_suspend __PROTO((void));
-static void term_close_output __PROTO((void));
+static void term_suspend(void);
+static void term_close_output(void);
 
-static void null_linewidth __PROTO((double));
-static void do_point __PROTO((unsigned int x, unsigned int y, int number));
-static void do_pointsize __PROTO((double size));
-static void line_and_point __PROTO((unsigned int x, unsigned int y, int number));
-static void do_arrow __PROTO((unsigned int sx, unsigned int sy, unsigned int ex, unsigned int ey, int head));
-static void null_dashtype __PROTO((int type, t_dashtype *custom_dash_pattern));
+static void null_linewidth(double);
+static void do_point(unsigned int x, unsigned int y, int number);
+static void do_pointsize(double size);
+static void line_and_point(unsigned int x, unsigned int y, int number);
+static void do_arrow(unsigned int sx, unsigned int sy, unsigned int ex, unsigned int ey, int headstyle);
+static void null_dashtype(int type, t_dashtype *custom_dash_pattern);
 
-static int null_text_angle __PROTO((int ang));
-static int null_justify_text __PROTO((enum JUSTIFY just));
-static int null_scale __PROTO((double x, double y));
-static void null_layer __PROTO((t_termlayer layer));
-static int null_set_font __PROTO((const char *font));
-static void null_set_color __PROTO((struct t_colorspec *colorspec));
-static void options_null __PROTO((void));
-static void graphics_null __PROTO((void));
-static void UNKNOWN_null __PROTO((void));
-static void MOVE_null __PROTO((unsigned int, unsigned int));
-static void LINETYPE_null __PROTO((int));
-static void PUTTEXT_null __PROTO((unsigned int, unsigned int, const char *));
+static int null_text_angle(int ang);
+static int null_justify_text(enum JUSTIFY just);
+static int null_scale(double x, double y);
+static void null_layer(t_termlayer layer);
+static int null_set_font(const char *font);
+static void null_set_color(struct t_colorspec *colorspec);
+static void options_null(void);
+static void graphics_null(void);
+static void UNKNOWN_null(void);
+static void MOVE_null(unsigned int, unsigned int);
+static void LINETYPE_null(int);
+static void PUTTEXT_null(unsigned int, unsigned int, const char *);
 
-static int strlen_tex __PROTO((const char *));
+static int strlen_tex(const char *);
 
-static char *stylefont __PROTO((const char *fontname, TBOOLEAN isbold, TBOOLEAN isitalic));
+static char *stylefont(const char *fontname, TBOOLEAN isbold, TBOOLEAN isitalic);
 
 /* Used by terminals and by shared routine parse_term_size() */
 typedef enum {
@@ -251,20 +246,14 @@ typedef enum {
     INCHES,
     CM
 } size_units;
-static size_units parse_term_size __PROTO((float *xsize, float *ysize, size_units def_units));
+static size_units parse_term_size(float *xsize, float *ysize, size_units def_units);
 
 
 #ifdef VMS
-char *vms_init();
-void vms_reset();
-void term_mode_tek();
-void term_mode_native();
-void term_pasthru();
-void term_nopasthru();
-void fflush_binary();
-# define FOPEN_BINARY(file) fopen(file, "wb", "rfm=fix", "bls=512", "mrs=512")
-#else /* !VMS */
+# include "vms.h"
+#else
 # define FOPEN_BINARY(file) fopen(file, "wb")
+# define fflush_binary()
 #endif /* !VMS */
 
 #if defined(MSDOS) || defined(_WIN32)
@@ -366,7 +355,7 @@ term_set_output(char *dest)
 #if defined(PIPES)
 	if (*dest == '|') {
 	    restrict_popen();
-#ifdef _WIN32
+#if defined(_WIN32 ) || defined(MSDOS)
 	    if (term && (term->flags & TERM_BINARY))
 		f = popen(dest + 1, "wb");
 	    else
@@ -527,6 +516,9 @@ term_start_plot()
 	term_suspended = FALSE;
     }
 
+    if (multiplot)
+	multiplot_count++;
+
     /* Sync point for epslatex text positioning */
     (*term->layer)(TERM_LAYER_RESET);
 
@@ -561,17 +553,19 @@ term_end_plot()
     } else {
 	multiplot_next();
     }
+
 #ifdef VMS
     if (opened_binary)
 	fflush_binary();
     else
 #endif /* VMS */
-
 	(void) fflush(gpoutfile);
 
 #ifdef USE_MOUSE
-    recalc_statusline();
-    update_ruler();
+    if (term->set_ruler) {
+	recalc_statusline();
+	update_ruler();
+    }
 #endif
 }
 
@@ -654,7 +648,6 @@ term_apply_lp_properties(struct lp_style_type *lp)
     (*term->linewidth) (lp->l_width);
 
     /* LT_DEFAULT (used only by "set errorbars"?) means don't change it */
-    /* FIXME: If this causes problems, test also for LP_ERRORBAR_SET    */
     if (lt == LT_DEFAULT)
 	;
     else
@@ -663,7 +656,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
     /* linetype < 0 (e.g. LT_BACKGROUND, LT_NODRAW) means some special */
     /* category that will be handled directly by term->linetype().     */
     /* linetype > 0 is now redundant. It used to encode both a color   */
-    /* and a dash pattern.  Now we have separate mechanisms for those. */ 
+    /* and a dash pattern.  Now we have separate mechanisms for those. */
     if (LT_COLORFROMCOLUMN < lt && lt < 0)
 	(*term->linetype) (lt);
     else if (term->set_color == null_set_color) {
@@ -672,6 +665,12 @@ term_apply_lp_properties(struct lp_style_type *lp)
     } else /* All normal lines will be solid unless a dashtype is given */
 	(*term->linetype) (LT_SOLID);
 
+    /* Version 5.3
+     * If the line is not wanted at all, setting dashtype and color can only hurt
+     */
+    if (lt == LT_NODRAW)
+	return;
+
     /* Apply dashtype or user-specified dash pattern, which may override  */
     /* the terminal-specific dot/dash pattern belonging to this linetype. */
     if (lt == LT_AXIS)
@@ -762,7 +761,7 @@ term_check_multiplot_okay(TBOOLEAN f_interactive)
 
 void
 write_multiline(
-    unsigned int x, unsigned int y,
+    int x, int y,
     char *text,
     JUSTIFY hor,                /* horizontal ... */
     VERT_JUSTIFY vert,          /* ... and vertical just - text in hor direction despite angle */
@@ -983,7 +982,7 @@ do_arrow(
     int ey = (int)uey;
 
     struct termentry *t = term;
-    float len_tic = ((double) (t->h_tic + t->v_tic)) / 2.0;
+    double len_tic = ((double) (t->h_tic + t->v_tic)) / 2.0;
     /* average of tic sizes */
     /* (dx,dy) : vector from end to start */
     double dx = sx - ex;
@@ -992,10 +991,8 @@ do_arrow(
     gpiPoint head_points[5];
     int xm = 0, ym = 0;
     BoundingBox *clip_save;
-    t_arrow_head head = (t_arrow_head)((headstyle < 0) ? -headstyle : headstyle);
-	/* negative headstyle means draw heads only, no shaft */
 
-    /* The arrow shaft was clipped already in do_clip_arrow() but we still */
+    /* The arrow shaft was clipped already in draw_clip_arrow() but we still */
     /* need to clip the head here. */
     clip_save = clip_area;
     if (term->flags & TERM_CAN_CLIP)
@@ -1007,7 +1004,7 @@ do_arrow(
      * Draw no head for arrows with length = 0, or, to be more specific,
      * length < DBL_EPSILON, because len_arrow will almost always be != 0.
      */
-    if ((head != NOHEAD) && fabs(len_arrow) >= DBL_EPSILON) {
+    if ((headstyle & BOTH_HEADS) != NOHEAD && fabs(len_arrow) >= DBL_EPSILON) {
 	int x1, y1, x2, y2;
 	if (curr_arrow_headlength <= 0) {
 	    /* An arrow head with the default size and angles */
@@ -1054,7 +1051,7 @@ do_arrow(
 	    ym = (int) (dy2 + backlen*effective_length * sin( phi + beta ));
 	}
 
-	if ((head & END_HEAD) && !clip_point(ex, ey)) {
+	if ((headstyle & END_HEAD) && !clip_point(ex, ey)) {
 	    head_points[0].x = ex + xm;
 	    head_points[0].y = ey + ym;
 	    head_points[1].x = ex + x1;
@@ -1065,22 +1062,24 @@ do_arrow(
 	    head_points[3].y = ey + y2;
 	    head_points[4].x = ex + xm;
 	    head_points[4].y = ey + ym;
-	    if (curr_arrow_headfilled >= AS_FILLED) {
-		/* draw filled forward arrow head */
-		head_points->style = FS_OPAQUE;
-		if (t->filled_polygon)
-		    (*t->filled_polygon) (5, head_points);
-	    }
-	    /* draw outline of forward arrow head */
-	    if (curr_arrow_headfilled == AS_NOFILL) {
-		draw_clip_polygon(3, head_points+1);
-	    } else if (curr_arrow_headfilled != AS_NOBORDER) {
-		draw_clip_polygon(5, head_points);
+	    if (!((headstyle & SHAFT_ONLY))) {
+		if (curr_arrow_headfilled >= AS_FILLED) {
+		    /* draw filled forward arrow head */
+		    head_points->style = FS_OPAQUE;
+		    if (t->filled_polygon)
+			(*t->filled_polygon) (5, head_points);
+		}
+		/* draw outline of forward arrow head */
+		if (curr_arrow_headfilled == AS_NOFILL) {
+		    draw_clip_polygon(3, head_points+1);
+		} else if (curr_arrow_headfilled != AS_NOBORDER) {
+		    draw_clip_polygon(5, head_points);
+		}
 	    }
 	}
 
 	/* backward arrow head */
-	if ((head & BACKHEAD) && !clip_point(sx,sy)) {
+	if ((headstyle & BACKHEAD) && !clip_point(sx,sy)) {
 	    head_points[0].x = sx - xm;
 	    head_points[0].y = sy - ym;
 	    head_points[1].x = sx - x1;
@@ -1091,52 +1090,53 @@ do_arrow(
 	    head_points[3].y = sy - y2;
 	    head_points[4].x = sx - xm;
 	    head_points[4].y = sy - ym;
-	    if (curr_arrow_headfilled >= AS_FILLED) {
-		/* draw filled backward arrow head */
-		head_points->style = FS_OPAQUE;
-		if (t->filled_polygon)
-		    (*t->filled_polygon) (5, head_points);
-	    }
-	    /* draw outline of backward arrow head */
-	    if (curr_arrow_headfilled == AS_NOFILL) {
-		draw_clip_polygon(3, head_points+1);
-	    } else if (curr_arrow_headfilled != AS_NOBORDER) {
-		draw_clip_polygon(5, head_points);
+	    if (!((headstyle & SHAFT_ONLY))) {
+		if (curr_arrow_headfilled >= AS_FILLED) {
+		    /* draw filled backward arrow head */
+		    head_points->style = FS_OPAQUE;
+		    if (t->filled_polygon)
+			(*t->filled_polygon) (5, head_points);
+		}
+		/* draw outline of backward arrow head */
+		if (curr_arrow_headfilled == AS_NOFILL) {
+		    draw_clip_polygon(3, head_points+1);
+		} else if (curr_arrow_headfilled != AS_NOBORDER) {
+		    draw_clip_polygon(5, head_points);
+		}
 	    }
 	}
     }
 
+    /* Adjust the length of the shaft so that it doesn't overlap the head */
+    if ((headstyle & BACKHEAD)
+    &&  (fabs(len_arrow) >= DBL_EPSILON) && (curr_arrow_headfilled != AS_NOFILL) ) {
+	sx -= xm;
+	sy -= ym;
+    }
+    if ((headstyle & END_HEAD)
+    &&  (fabs(len_arrow) >= DBL_EPSILON) && (curr_arrow_headfilled != AS_NOFILL) ) {
+	ex += xm;
+	ey += ym;
+    }
+
     /* Draw the line for the arrow. */
-    if (headstyle >= 0) {
-	if ((head & BACKHEAD)
-	&&  (fabs(len_arrow) >= DBL_EPSILON) && (curr_arrow_headfilled != AS_NOFILL) ) {
-	    sx -= xm;
-	    sy -= ym;
-	}
-	if ((head & END_HEAD)
-	&&  (fabs(len_arrow) >= DBL_EPSILON) && (curr_arrow_headfilled != AS_NOFILL) ) {
-	    ex += xm;
-	    ey += ym;
-	}
+    if (!((headstyle & HEADS_ONLY)))
 	draw_clip_line(sx, sy, ex, ey);
-    }
 
     /* Restore previous clipping box */
     clip_area = clip_save;
-
 }
 
-#ifdef EAM_OBJECTS
 /* Generic routine for drawing circles or circular arcs.          */
 /* If this feature proves useful, we can add a new terminal entry */
 /* point term->arc() to the API and let terminals either provide  */
-/* a private implemenation or use this generic one.               */
+/* a private implementation or use this generic one.               */
 
 void
 do_arc(
     int cx, int cy, /* Center */
     double radius, /* Radius */
-    double arc_start, double arc_end, /* Limits of arc in degress */
+    double arc_start, double arc_end, /* Limits of arc in degrees */
     int style, TBOOLEAN wedge)
 {
     gpiPoint vertex[250];
@@ -1155,7 +1155,7 @@ do_arc(
 	arc_end += 360.;
 
     /* Choose how finely to divide this arc into segments */
-    /* FIXME: INC=2 causes problems for gnuplot_x11 */
+    /* Note: INC=2 caused problems for gnuplot_x11 */
 #   define INC 3.
     segments = (arc_end - arc_start) / INC;
     if (segments < 1)
@@ -1196,8 +1196,6 @@ do_arc(
 	draw_clip_polygon(segments+1, vertex);
     }
 }
-#endif /* EAM_OBJECTS */
-
 
 
 #define TERM_PROTO
@@ -1236,7 +1234,7 @@ null_justify_text(enum JUSTIFY just)
 }
 
 
-/* 
+/*
  * Deprecated terminal function (pre-version 3)
  */
 static int
@@ -1476,6 +1474,13 @@ change_term(const char *origname, int length)
 	length = 8;
     }
 #endif
+#ifdef HAVE_LIBGD
+    /* To allow "set term sixel" as short for "set term sixelgd" */
+    if (!strncmp(origname, "sixel", length)) {
+	name = "sixelgd";
+	length = 7;
+    }
+#endif
 
     for (i = 0; i < TERMCOUNT; i++) {
 	if (!strncmp(name, term_tbl[i].name, length)) {
@@ -1546,7 +1551,7 @@ void
 init_terminal()
 {
     char *term_name = DEFAULTTERM;
-#if (defined(MSDOS) && !defined(_WIN32)) || defined(SUN) || defined(X11)
+#if defined(__BEOS__) || defined(X11)
     char *env_term = NULL;      /* from TERM environment var */
 #endif
 #ifdef X11
@@ -1585,11 +1590,6 @@ init_terminal()
 	    term_name = "be";
 #endif /* BeOS */
 
-#if defined(WXWIDGETS) && defined(_WIN32)
-	if (term_name == (char *) NULL)
-	    term_name = "wxt";
-#endif
-
 #ifdef QTTERM
 	if (term_name == (char *) NULL)
 	    term_name = "qt";
@@ -1645,7 +1645,7 @@ init_terminal()
 	    term_name = "pm";
 #endif /*OS2 */
 
-/* set linux terminal only if LINUX_setup was successfull, if we are on X11
+/* set linux terminal only if LINUX_setup was successful, if we are on X11
    LINUX_setup has failed, also if we are logged in by network */
 #ifdef LINUXVGA
 	if (LINUX_graphics_allowed)
@@ -1751,21 +1751,24 @@ test_term()
      * Textbox fill shows true size, surrounding box shows the generic estimate
      * used to reserve space during plot layout.
      */
-#ifdef EAM_BOXED_TEXT
     if (TRUE) {
 	struct text_label sample = EMPTY_LABELSTRUCT;
-	struct textbox_style save_opts = textbox_opts;
+	struct textbox_style save_opts = textbox_opts[0];
+	textbox_style *textbox = &textbox_opts[0];
 	sample.text = "12345678901234567890";
 	sample.pos = CENTRE;
-	sample.boxed = 1;
-	textbox_opts.opaque = TRUE;
-	textbox_opts.noborder = TRUE;
-	textbox_opts.fillcolor.type = TC_RGB;
-	textbox_opts.fillcolor.lt = 0xccccee;
+	sample.boxed = -1;
+	textbox->opaque = TRUE;
+	textbox->noborder = TRUE;
+	textbox->fillcolor.type = TC_RGB;
+	textbox->fillcolor.lt = 0xccccee;
+	/* disable extra space around text */
+	textbox->xmargin = 0;
+	textbox->ymargin = 0;
 
 	(*t->linetype) (LT_SOLID);
 	write_label(xmax_t/2, ymax_t/2, &sample);
-	textbox_opts = save_opts;
+	textbox_opts[0] = save_opts;
 
 	sample.boxed = 0;
 	sample.text = "true vs. estimated text dimensions";
@@ -1779,13 +1782,12 @@ test_term()
 	(*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
 	closepath();
     }
-#endif
 
     /* Test for enhanced text */
     (*t->linetype) (LT_BLACK);
     if (t->flags & TERM_ENHANCED_TEXT) {
 	char *tmptext1 =   "Enhanced text:   {x@_{0}^{n+1}}";
-	char *tmptext2 = "&{Enhanced text:  }{/:Bold Bold}{/:Italic  Italic}";  
+	char *tmptext2 = "&{Enhanced text:  }{/:Bold Bold}{/:Italic  Italic}";
 	(*t->put_text) (x0 + xmax_t * 0.5, y0 + ymax_t * 0.40, tmptext1);
 	(*t->put_text) (x0 + xmax_t * 0.5, y0 + ymax_t * 0.35, tmptext2);
 	(*t->set_font)("");
@@ -1898,7 +1900,7 @@ test_term()
     y = y0 + yl;
 
     for (i=1; i<7; i++) {
-	(*t->linewidth) ((float)(i)); (*t->linetype)(LT_BLACK);
+	(*t->linewidth) ((double)(i)); (*t->linetype)(LT_BLACK);
 	(*t->move) (x, y); (*t->vector) (x+xl, y);
 	sprintf(label,"  lw %1d", i);
 	(*t->put_text) (x+xl, y, label);
@@ -1912,11 +1914,11 @@ test_term()
     yl = ymax_t / 25;
     x = x0 + xmax_t * .3;
     y = y0 + yl;
-    
+
     for (i=0; i<5; i++) {
  	(*t->linewidth) (1.0);
 	(*t->linetype) (LT_SOLID);
-	(*t->dashtype) (i, NULL); 
+	(*t->dashtype) (i, NULL);
 	(*t->set_color)(&black);
 	(*t->move) (x, y); (*t->vector) (x+xl, y);
 	sprintf(label,"  dt %1d", i+1);
@@ -1930,7 +1932,7 @@ test_term()
     y = y0;
     xl = xmax_t / 40;
     yl = ymax_t / 8;
-    (*t->linewidth) ((float)(1));
+    (*t->linewidth) (1.0);
     (*t->linetype)(LT_BLACK);
     (*t->justify_text) (CENTRE);
     (*t->put_text)(x+xl*7, y + yl+t->v_char*1.5, "pattern fill");
@@ -1992,183 +1994,6 @@ test_term()
     term_end_plot();
 }
 
-
-#ifdef VMS
-/* these are needed to modify terminal characteristics */
-# ifndef VWS_XMAX
-   /* avoid duplicate warning; VWS includes these */
-#  include <descrip.h>
-#  include <ssdef.h>
-# endif                         /* !VWS_MAX */
-# include <iodef.h>
-# include <ttdef.h>
-# include <tt2def.h>
-# include <dcdef.h>
-# include <stat.h>
-# include <fab.h>
-/* If you use WATCOM C or a very strict ANSI compiler, you may have to
- * delete or comment out the following 3 lines: */
-# ifndef TT2$M_DECCRT3          /* VT300 not defined as of VAXC v2.4 */
-#  define TT2$M_DECCRT3 0X80000000
-# endif
-static unsigned short chan;
-static int old_char_buf[3], cur_char_buf[3];
-$DESCRIPTOR(sysoutput_desc, "SYS$OUTPUT");
-
-/* Look first for decw$display (decterms do regis).  Determine if we
- * have a regis terminal and save terminal characteristics */
-char *
-vms_init()
-{
-    /* Save terminal characteristics in old_char_buf and
-       initialise cur_char_buf to current settings. */
-    int i;
-#ifdef X11
-    if (getenv("DECW$DISPLAY"))
-	return ("x11");
-#endif
-    atexit(vms_reset);
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    sys$qiow(0, chan, IO$_SENSEMODE, 0, 0, 0, old_char_buf, 12, 0, 0, 0, 0);
-    for (i = 0; i < 3; ++i)
-	cur_char_buf[i] = old_char_buf[i];
-    sys$dassgn(chan);
-
-    /* Test if terminal is regis */
-    if ((cur_char_buf[2] & TT2$M_REGIS) == TT2$M_REGIS)
-	return ("regis");
-    return (NULL);
-}
-
-/* set terminal to original state */
-void
-vms_reset()
-{
-    int i;
-
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, old_char_buf, 12, 0, 0, 0, 0);
-    for (i = 0; i < 3; ++i)
-	cur_char_buf[i] = old_char_buf[i];
-    sys$dassgn(chan);
-}
-
-/* set terminal mode to tektronix */
-void
-term_mode_tek()
-{
-    long status;
-
-    if (gpoutfile != stdout)
-	return;                 /* don't modify if not stdout */
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    cur_char_buf[0] = 0x004A0000 | DC$_TERM | (TT$_TEK401X << 8);
-    cur_char_buf[1] = (cur_char_buf[1] & 0x00FFFFFF) | 0x18000000;
-
-    cur_char_buf[1] &= ~TT$M_CRFILL;
-    cur_char_buf[1] &= ~TT$M_ESCAPE;
-    cur_char_buf[1] &= ~TT$M_HALFDUP;
-    cur_char_buf[1] &= ~TT$M_LFFILL;
-    cur_char_buf[1] &= ~TT$M_MECHFORM;
-    cur_char_buf[1] &= ~TT$M_NOBRDCST;
-    cur_char_buf[1] &= ~TT$M_NOECHO;
-    cur_char_buf[1] &= ~TT$M_READSYNC;
-    cur_char_buf[1] &= ~TT$M_REMOTE;
-    cur_char_buf[1] |= TT$M_LOWER;
-    cur_char_buf[1] |= TT$M_TTSYNC;
-    cur_char_buf[1] |= TT$M_WRAP;
-    cur_char_buf[1] &= ~TT$M_EIGHTBIT;
-    cur_char_buf[1] &= ~TT$M_MECHTAB;
-    cur_char_buf[1] &= ~TT$M_SCOPE;
-    cur_char_buf[1] |= TT$M_HOSTSYNC;
-
-    cur_char_buf[2] &= ~TT2$M_APP_KEYPAD;
-    cur_char_buf[2] &= ~TT2$M_BLOCK;
-    cur_char_buf[2] &= ~TT2$M_DECCRT3;
-    cur_char_buf[2] &= ~TT2$M_LOCALECHO;
-    cur_char_buf[2] &= ~TT2$M_PASTHRU;
-    cur_char_buf[2] &= ~TT2$M_REGIS;
-    cur_char_buf[2] &= ~TT2$M_SIXEL;
-    cur_char_buf[2] |= TT2$M_BRDCSTMBX;
-    cur_char_buf[2] |= TT2$M_EDITING;
-    cur_char_buf[2] |= TT2$M_INSERT;
-    cur_char_buf[2] |= TT2$M_PRINTER;
-    cur_char_buf[2] &= ~TT2$M_ANSICRT;
-    cur_char_buf[2] &= ~TT2$M_AVO;
-    cur_char_buf[2] &= ~TT2$M_DECCRT;
-    cur_char_buf[2] &= ~TT2$M_DECCRT2;
-    cur_char_buf[2] &= ~TT2$M_DRCS;
-    cur_char_buf[2] &= ~TT2$M_EDIT;
-    cur_char_buf[2] |= TT2$M_FALLBACK;
-
-    status = sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, cur_char_buf, 12, 0, 0, 0, 0);
-    if (status == SS$_BADPARAM) {
-	/* terminal fallback utility not installed on system */
-	cur_char_buf[2] &= ~TT2$M_FALLBACK;
-	sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, cur_char_buf, 12, 0, 0, 0, 0);
-    } else {
-	if (status != SS$_NORMAL)
-	    lib$signal(status, 0, 0);
-    }
-    sys$dassgn(chan);
-}
-
-/* set terminal mode back to native */
-void
-term_mode_native()
-{
-    int i;
-
-    if (gpoutfile != stdout)
-	return;                 /* don't modify if not stdout */
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, old_char_buf, 12, 0, 0, 0, 0);
-    for (i = 0; i < 3; ++i)
-	cur_char_buf[i] = old_char_buf[i];
-    sys$dassgn(chan);
-}
-
-/* set terminal mode pasthru */
-void
-term_pasthru()
-{
-    if (gpoutfile != stdout)
-	return;                 /* don't modify if not stdout */
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    cur_char_buf[2] |= TT2$M_PASTHRU;
-    sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, cur_char_buf, 12, 0, 0, 0, 0);
-    sys$dassgn(chan);
-}
-
-/* set terminal mode nopasthru */
-void
-term_nopasthru()
-{
-    if (gpoutfile != stdout)
-	return;                 /* don't modify if not stdout */
-    sys$assign(&sysoutput_desc, &chan, 0, 0);
-    cur_char_buf[2] &= ~TT2$M_PASTHRU;
-    sys$qiow(0, chan, IO$_SETMODE, 0, 0, 0, cur_char_buf, 12, 0, 0, 0, 0);
-    sys$dassgn(chan);
-}
-
-void
-fflush_binary()
-{
-    typedef short int INT16;    /* signed 16-bit integers */
-    INT16 k;            /* loop index */
-
-    if (gpoutfile != stdout) {
-	/* Stupid VMS fflush() raises error and loses last data block
-	   unless it is full for a fixed-length record binary file.
-	   Pad it here with NULL characters. */
-	for (k = (INT16) ((*gpoutfile)->_cnt); k > 0; --k)
-	    putc('\0', gpoutfile);
-	fflush(gpoutfile);
-    }
-}
-#endif /* VMS */
-
 /*
  * This is an abstraction of the enhanced text mode originally written
  * for the postscript terminal driver by David Denholm and Matt Heffron.
@@ -2207,6 +2032,9 @@ fflush_binary()
 void
 do_enh_writec(int c)
 {
+    /* Guard against buffer overflow */
+    if (enhanced_cur_text >= ENHANCED_TEXT_MAX)
+	return;
     /* note: c is meant to hold a char, but is actually an int, for
      * the same reasons applying to putc() and friends */
     *enhanced_cur_text++ = c;
@@ -2262,7 +2090,7 @@ enhanced_recursion(
     }
 
     while (*p) {
-	float shift;
+	double shift;
 
 	/*
 	 * EAM Jun 2009 - treating bytes one at a time does not work for multibyte
@@ -2318,7 +2146,7 @@ enhanced_recursion(
 		const char *end_of_fontname = NULL;
 		char *localfontname = NULL;
 		char ch;
-		float f = fontsize, ovp;
+		double f = fontsize, ovp;
 
 		/* Mar 2014 - this will hold "fontfamily{:Italic}{:Bold}" */
 		char *styledfontname = NULL;
@@ -2333,7 +2161,7 @@ enhanced_recursion(
 		/* get vertical offset (if present) for overprinted text */
 		if (overprint == 2) {
 		    char *end;
-		    ovp = (float)strtod(p,&end);
+		    ovp = strtod(p,&end);
 		    p = end;
 		    if (term->flags & TERM_IS_POSTSCRIPT)
 			base = ovp*f;
@@ -2361,7 +2189,7 @@ enhanced_recursion(
 			    ++p;
 			if (*p != *start_of_fontname) {
 			    int_warn(NO_CARET, "cannot interpret font name %s", start_of_fontname);
-			    p = start_of_fontname;
+			    break;
 			}
 			start_of_fontname++;
 			end_of_fontname = p++;
@@ -2380,7 +2208,7 @@ enhanced_recursion(
 			    char *end;
 			    p++;
 			    ENH_DEBUG(("Calling strtod(\"%s\") ...", p));
-			    f = (float)strtod(p, &end);
+			    f = strtod(p, &end);
 			    p = end;
 			    ENH_DEBUG(("Returned %.1f and \"%s\"\n", f, p));
 
@@ -2395,7 +2223,7 @@ enhanced_recursion(
 			    char *end;
 			    p++;
 			    ENH_DEBUG(("Calling strtod(\"%s\") ...", p));
-			    f = (float)strtod(p, &end);
+			    f = strtod(p, &end);
 			    p = end;
 			    ENH_DEBUG(("Returned %.1f and \"%s\"\n", f, p));
 
@@ -2510,14 +2338,48 @@ enhanced_recursion(
 	    /*}}}*/
 
 	case '\\'  :
-	    /*{{{  Enhanced mode always uses \xyz as an octal character representation
-		   but each terminal type must give us the actual output format wanted.
-		   pdf.trm wants the raw character code, which is why we use strtol();
-		   most other terminal types want some variant of "\\%o". */
+	    /*{{{  various types of escape sequences, some context-dependent */
+	    (term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
+
+	    /*     Unicode represented as \U+hhhhh where hhhhh is hexadecimal code point.
+	     *     For UTF-8 encoding we translate hhhhh to a UTF-8 byte sequence and
+	     *     output the bytes one by one.
+	     */
+	    if (p[1] == 'U' && p[2] == '+') {
+		if (encoding == S_ENC_UTF8) {
+		    uint32_t codepoint;
+		    unsigned char utf8char[8];
+		    int i, length;
+		    if (strlen(&(p[3])) < 4)
+			break;
+		    if (sscanf(&(p[3]), "%5x", &codepoint) != 1)
+			break;
+		    length = ucs4toutf8(codepoint, utf8char);
+		    p += (codepoint > 0xFFFF) ? 7 : 6;
+		    for (i=0; i<length; i++)
+			(term->enhanced_writec)(utf8char[i]);
+		    break;
+		}
+
+	    /*     FIXME: non-utf8 environments not yet supported.
+	     *     Note that some terminals may have an alternative way to handle unicode
+	     *     escape sequences that is not dependent on encoding.
+	     *     E.g. svg and html output could convert to xml sequences &#xhhhh;
+	     *     For these cases we must retain the leading backslash so that the
+	     *     unicode escape sequence can be recognized by the terminal driver.
+	     */
+		(term->enhanced_writec)(p[0]);
+		break;
+	    }
+
+	    /* Enhanced mode always uses \xyz as an octal character representation
+	     * but each terminal type must give us the actual output format wanted.
+	     * pdf.trm wants the raw character code, which is why we use strtol();
+	     * most other terminal types want some variant of "\\%o".
+	     */
 	    if (p[1] >= '0' && p[1] <= '7') {
 		char *e, escape[16], octal[4] = {'\0','\0','\0','\0'};
 
-		(term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
 		octal[0] = *(++p);
 		if (p[1] >= '0' && p[1] <= '7') {
 		    octal[1] = *(++p);
@@ -2529,24 +2391,25 @@ enhanced_recursion(
 		    (term->enhanced_writec)(*e);
 		}
 		break;
-	    /* This was the original (prior to version 4) enhanced text code specific */
-	    /* to the reserved characters of PostScript.  Some of it was mis-applied  */
-	    /* to other terminal types until fixed in Mar 2012.                       */
-	    } else if (term->flags & TERM_IS_POSTSCRIPT) {
+	    }
+
+	    /* This was the original (prior to version 4) enhanced text code specific
+	     * to the reserved characters of PostScript.
+	     */
+	    if (term->flags & TERM_IS_POSTSCRIPT) {
 		if (p[1]=='\\' || p[1]=='(' || p[1]==')') {
-		    (term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
 		    (term->enhanced_writec)('\\');
 		} else if (strchr("^_@&~{}",p[1]) == NULL) {
-		    (term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
 		    (term->enhanced_writec)('\\');
 		    (term->enhanced_writec)('\\');
 		    break;
 		}
 	    }
+
+	    /* Step past the backslash character in the input stream */
 	    ++p;
 
-	    /* HBB 20030122: Avoid broken output if there's a \
-	     * exactly at the end of the line */
+	    /* HBB: Avoid broken output if there's a \ exactly at the end of the line */
 	    if (*p == '\0') {
 		int_warn(NO_CARET, "enhanced text parser -- spurious backslash");
 		break;
@@ -2555,13 +2418,15 @@ enhanced_recursion(
 	    /* SVG requires an escaped '&' to be passed as something else */
 	    /* FIXME: terminal-dependent code does not belong here */
 	    if (*p == '&' && encoding == S_ENC_DEFAULT && !strcmp(term->name, "svg")) {
-		(term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
 		(term->enhanced_writec)('\376');
 		break;
 	    }
 
-	    /* just go and print it (fall into the 'default' case) */
+	    /* print the character following the backslash */
+	    (term->enhanced_writec)(*p);
+	    break;
 	    /*}}}*/
+
 	default:
 	    /*{{{  print it */
 	    (term->enhanced_open)(fontname, fontsize, base, widthflag, showflag, overprint);
@@ -2592,11 +2457,12 @@ enhanced_recursion(
 char *
 stylefont(const char *fontname, TBOOLEAN isbold, TBOOLEAN isitalic)
 {
-    char *div;
+    int div;
     char *markup = gp_alloc( strlen(fontname) + 16, "font markup");
     strcpy(markup, fontname);
-    if ((div = strchr(markup,':')))
-	*div = '\0';
+    /* base font name can be followed by ,<size> or :Variant */
+    if ((div = strcspn(markup,",:")))
+	markup[div] = '\0';
     if (isbold)
 	strcat(markup, ":Bold");
     if (isitalic)
@@ -2629,7 +2495,8 @@ enh_err_check(const char *str)
 int
 estimate_strlen(const char *text)
 {
-int len;
+    int len;
+    char *s;
 
     if ((term->flags & TERM_IS_LATEX))
 	len = strlen_tex(text);
@@ -2641,9 +2508,17 @@ int len;
 	term = &ENHest;
 	term->put_text(0,0,text);
 	len = term->xmax;
-	FPRINTF((stderr,"Estimating length %d height %g for enhanced text string \"%s\"\n",
-		len, (double)(term->ymax)/10., text));
 	term = tsave;
+	/* Assume that unicode escape sequences  \U+xxxx will generate a single character */
+	/* ENHest_plaintext is filled in by the put_text() call to estimate.trm           */
+	s = ENHest_plaintext;
+	while ((s = contains_unicode(s)) != NULL) {
+	    len -= 6;
+	    s += 6;
+	}
+	FPRINTF((stderr,"Estimating length %d height %g for enhanced text \"%s\"",
+		len, (double)(term->ymax)/10., text));
+	FPRINTF((stderr,"  plain text \"%s\"\n", ENHest_plaintext));
     } else if (encoding == S_ENC_UTF8)
 	len = strwidth_utf8(text);
     else
@@ -2653,7 +2528,7 @@ int len;
     return len;
 }
 
-/* 
+/*
  * Use estimate.trm to mock up a non-enhanced approximation of the
  * original string.
  */
@@ -2669,17 +2544,6 @@ estimate_plaintext(char *enhancedtext)
 void
 ignore_enhanced(TBOOLEAN flag)
 {
-#if (0)
-    /* Apr 2018: This code was introduced long ago (2005; Bug #266) to address
-     * a glitch in the postscript terminal that left the last-used font in
-     * an enhanced text string active afterwards.
-     * We now deal with this in ENHPS_put_text() instead.
-     */
-    if (flag && !ignore_enhanced_text) {
-	ignore_enhanced_text = TRUE;
-	term->set_font("");
-    }
-#endif
     ignore_enhanced_text = flag;
 }
 
@@ -3003,7 +2867,7 @@ strlen_tex(const char *str)
 
 /* The check for asynchronous events such as hotkeys and mouse clicks is
  * normally done in term->waitforinput() while waiting for the next input
- * from the command line.  If input is currently coming from a file or 
+ * from the command line.  If input is currently coming from a file or
  * pipe instead, as with a "load" command, then this path would not be
  * triggered automatically and these events would back up until input
  * returned to the command line.  These code paths can explicitly call
@@ -3031,3 +2895,28 @@ check_for_mouse_events()
     }
 #endif
 }
+
+char *
+escape_reserved_chars(const char *str, const char *reserved)
+{
+    int i;
+    char *escaped_str;
+    int newsize = strlen(str);
+
+	/* Count number of reserved characters */
+	for (i = 0; str[i] != '\0'; i++) {
+	    if (strchr(reserved, str[i]))
+		newsize++;
+	}
+	escaped_str = gp_alloc(newsize + 1, NULL);
+
+	/* Prefix each reserved character with a backslash */
+	for (i = 0, newsize = 0; str[i] != '\0'; i++) {
+	    if (strchr(reserved, str[i]))
+		escaped_str[newsize++] = '\\';
+	    escaped_str[newsize++] = str[i];
+	}
+	escaped_str[newsize] = '\0';
+
+    return escaped_str;
+}
